package gregapi.tileentity.base;

import static gregapi.data.CS.*;
import gregapi.block.multitileentity.IMultiTileEntity.IMTE_BreakBlock;
import gregapi.block.multitileentity.IMultiTileEntity.IMTE_OnBlockExploded;
import gregapi.block.multitileentity.MultiTileEntityRegistry;
import gregapi.tileentity.ITileEntityInventoryGUI;
import gregapi.util.OM;
import gregapi.util.ST;
import gregapi.util.UT;
import net.minecraft.entity.item.EntityItem;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.inventory.IInventory;
import net.minecraft.item.ItemStack;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.nbt.NBTTagList;
import net.minecraft.world.Explosion;

/**
 * @author Gregorius Techneticies
 */
public abstract class TileEntityBase05Inventories extends TileEntityBase04MultiTileEntities implements IInventory, ITileEntityInventoryGUI, IMTE_OnBlockExploded, IMTE_BreakBlock {
	private ItemStack[] mInventory = ZL_ITEMSTACK;
	
	public boolean mInventoryChanged = F;
	
	@Override
	public void readFromNBT2(NBTTagCompound aNBT) {
		// Standard readFromNBT process to load Inventory.
		mInventory = getDefaultInventory(aNBT);
		if (mInventory != null && mInventory.length > 0) {
			NBTTagList tList = aNBT.getTagList(NBT_INV_LIST, 10);
			for (int i = 0; i < tList.tagCount(); i++) {
				NBTTagCompound tNBT = tList.getCompoundTagAt(i);
				int tSlot = tNBT.getShort("s");
				if (tSlot >= 0 && tSlot < mInventory.length) mInventory[tSlot] = ST.load(tNBT);
			}
		}
	}
	
	@Override
	public void writeToNBT2(NBTTagCompound aNBT) {
		if (mInventory != null && mInventory.length > 0) {
			NBTTagList tList = new NBTTagList();
			for (short tSlot = 0; tSlot < mInventory.length; tSlot++) if (mInventory[tSlot] != null) tList.appendTag(UT.NBT.getNBTShort(ST.save(mInventory[tSlot]), "s", tSlot));
			aNBT.setTag(NBT_INV_LIST, tList);
		}
	}
	
	public ItemStack[] getDefaultInventory(NBTTagCompound aNBT) {
		short tSize = aNBT.getShort(NBT_INV_SIZE);
		return tSize > 0 ? new ItemStack[tSize] : ZL_ITEMSTACK;
	}
	
	@Override
	public void onTickResetChecks(long aTimer, boolean aIsServerSide) {
		super.onTickResetChecks(aTimer, aIsServerSide);
		mInventoryChanged = F;
	}
	
	public final ItemStack slot(int aIndex, ItemStack aStack) {return mInventory[aIndex] = aStack;}
	public final ItemStack slot(int aIndex) {return mInventory[aIndex];}
	public final boolean slotHas(int aIndex) {return mInventory[aIndex] != null;}
	public final boolean invempty() {for (int i = 0; i < mInventory.length; i++) if (mInventory[i] != null) return F; return T;}
	public final NBTTagCompound slotNBT(int aIndex) {return mInventory[aIndex] != null ? mInventory[aIndex].getTagCompound() : null;}
	
	@Override protected void updateInventory() {mInventoryChanged = T;}
	@Override public boolean isUseableByPlayer(EntityPlayer aPlayer) {return !isDead() && allowInteraction(aPlayer) && aPlayer.getDistanceSq(xCoord + 0.5D, yCoord + 0.5D, zCoord + 0.5D) <= 64D;}
	@Override public void openInventory() {/**/}
	@Override public void closeInventory() {/**/}
	@Override public int getInventoryStackLimit() {return 64;}
	@Override public void markDirty() {super.markDirty(); updateInventory();}
	@Override public ItemStack decrStackSize(int aSlot, int aDecrement) {updateInventory(); if (mInventory[aSlot] == null) return null; if (mInventory[aSlot].stackSize <= aDecrement) {ItemStack tStack = ST.copy(mInventory[aSlot]); if (allowZeroStacks(aSlot)) mInventory[aSlot].stackSize = 0; else mInventory[aSlot] = NI; return tStack;} ItemStack rStack = mInventory[aSlot].splitStack(aDecrement); if (mInventory[aSlot].stackSize <= 0 && !allowZeroStacks(aSlot)) mInventory[aSlot] = NI; return rStack;}
	@Override public ItemStack getStackInSlotOnClosing(int aSlot) {ItemStack rStack = mInventory[aSlot]; mInventory[aSlot] = null; return rStack;}
	@Override public ItemStack getStackInSlot(int aSlot) {return mInventory[aSlot];}
	@Override public String getInventoryName() {String rName = getCustomName(); if (UT.Code.stringValid(rName)) return rName; MultiTileEntityRegistry tRegistry = MultiTileEntityRegistry.getRegistry(getMultiTileEntityRegistryID()); return tRegistry==null?getClass().getName():tRegistry.getLocal(getMultiTileEntityID());}
	@Override public int getSizeInventory() {return mInventory==null?0:mInventory.length;}
	@Override public void setInventorySlotContents(int aSlot, ItemStack aStack) {updateInventory(); mInventory[aSlot] = OM.get(aStack);}
	@Override public boolean hasCustomInventoryName() {return getCustomName() != null;}
	@Override public boolean isItemValidForSlot(int aSlot, ItemStack aStack) {return T;}
	@Override public void onExploded(Explosion aExplosion) {for (int i = 0; i < mInventory.length; i++) if (worldObj.rand.nextInt(3) != 0) mInventory[i] = null; worldObj.setBlockToAir(xCoord, yCoord, zCoord);}
	public boolean allowZeroStacks(int aSlot) {return F;}
	public ItemStack[] getInventory() {return mInventory;}
	public void setInventory(ItemStack[] aInventory) {mInventory = aInventory;}
	public void removeAllDroppableNullStacks() {for (int i = 0; i < mInventory.length; i++) if (canDrop(i) && mInventory[i] != null && mInventory[i].stackSize <= 0) mInventory[i] = null;}
	
	public abstract boolean canDrop(int aSlot);
	
	// These Functions are intentionally duplicates of the Functions above.
	@Override public int getSizeInventoryGUI() {return mInventory==null?0:mInventory.length;}
	@Override public ItemStack getStackInSlotGUI(int aSlot) {return mInventory[aSlot];}
	@Override public ItemStack decrStackSizeGUI(int aSlot, int aDecrement) {updateInventory(); if (mInventory[aSlot] == null) return null; if (mInventory[aSlot].stackSize <= aDecrement) {ItemStack tStack = ST.copy(mInventory[aSlot]); if (allowZeroStacks(aSlot)) mInventory[aSlot].stackSize = 0; else mInventory[aSlot] = NI; return tStack;} ItemStack rStack = mInventory[aSlot].splitStack(aDecrement); if (mInventory[aSlot].stackSize <= 0 && !allowZeroStacks(aSlot)) mInventory[aSlot] = NI; return rStack;}
	@Override public ItemStack getStackInSlotOnClosingGUI(int aSlot) {ItemStack rStack = mInventory[aSlot]; mInventory[aSlot] = null; return rStack;}
	@Override public void setInventorySlotContentsGUI(int aSlot, ItemStack aStack) {updateInventory(); mInventory[aSlot] = OM.get(aStack);}
	@Override public String getInventoryNameGUI() {String rName = getCustomName(); if (UT.Code.stringValid(rName)) return rName; MultiTileEntityRegistry tRegistry = MultiTileEntityRegistry.getRegistry(getMultiTileEntityRegistryID()); return tRegistry==null?getClass().getName():tRegistry.getLocal(getMultiTileEntityID());}
	@Override public boolean hasCustomInventoryNameGUI() {return getCustomName() != null;}
	@Override public int getInventoryStackLimitGUI(int aSlot) {return getInventoryStackLimit();}
	@Override public void markDirtyGUI() {markDirty();}
	@Override public boolean isUseableByPlayerGUI(EntityPlayer aPlayer) {return !isDead() && allowInteraction(aPlayer) && aPlayer.getDistanceSq(xCoord + 0.5D, yCoord + 0.5D, zCoord + 0.5D) <= 64D;}
	@Override public void openInventoryGUI() {openInventory();}
	@Override public void closeInventoryGUI() {closeInventory();}
	@Override public boolean isItemValidForSlotGUI(int aSlot, ItemStack aStack) {return T;}
	
	@Override
	public boolean breakBlock() {
		if (isServerSide()) for (short i = 0; i < mInventory.length; i++) if (mInventory[i] != null && canDrop(i) && !ST.debug(mInventory[i])) {
			mInventory[i].stackSize = (int)UT.Code.bind_(0, 512 * Math.max(1, mInventory[i].getMaxStackSize()), mInventory[i].stackSize);
			while (mInventory[i].stackSize > Math.max(1, mInventory[i].getMaxStackSize())) {
				worldObj.spawnEntityInWorld(new EntityItem(worldObj, xCoord+0.5, yCoord+0.5, zCoord+0.5, ST.amount(Math.max(1, mInventory[i].getMaxStackSize()), mInventory[i])));
				mInventory[i].stackSize -= Math.max(1, mInventory[i].getMaxStackSize());
			}
			if (mInventory[i].stackSize > 0) worldObj.spawnEntityInWorld(new EntityItem(worldObj, xCoord+0.5, yCoord+0.5, zCoord+0.5, ST.copy(mInventory[i])));
		}
		return F;
	}
	
	public boolean addStackToSlot(int aIndex, ItemStack aStack) {
		if (ST.invalid(aStack)) return T;
		if (aIndex < 0 || aIndex >= getSizeInventory()) return F;
		ItemStack tStack = getStackInSlot(aIndex);
		if (ST.invalid(tStack)) {
			setInventorySlotContents(aIndex, aStack);
			return T;
		}
		aStack = OM.get_(aStack);
		if (ST.equal(tStack, aStack) && tStack.stackSize + aStack.stackSize <= Math.min(Math.max(1, tStack.getMaxStackSize()), getInventoryStackLimit())) {
			tStack.stackSize+=aStack.stackSize;
			updateInventory();
			return T;
		}
		return F;
	}
}